import md5 from 'md5';

enum AnnotationStatus {
    fixed=1,
    un_fixed=0,
}

enum LightStatus {
    un_light=0,
    half_light=1,
    light=2,
}

interface Step {
    bg: string,
    id: string,
    isActive: string,
    offsetX: number,
    offsetY: number,
    parentW: number,
    x: number,
    y: number,
    time: string,
    pre: string,
    tip: string,
    suffix: string,
    text: string,
    annotationStatus: AnnotationStatus,
    lightStatus: LightStatus
}

interface PlainData {
    url: string,
    images: string[],
    categories: string[],
    snapshots: string[],
    setting: any,
    steps: Step[],

    // TODO 废弃 SDK 不处理
    title?: string,
    version?: string,
    icon?: string,
    createAt?: number,
    description?: string,
    lastModified?: number,
}

type WebPageIds = {
    key: string, // 此数据的唯一标识符，一般为 URL，但也可能是hash值
    url: string, // 此条数据绑定的 URL
    urls: string[], // 此条数据绑定的 URL 集合
}

type WebPageTimes = {
    createAt: number,
    updateAt: number,
    lastSyncTime?: number, // 云盘最后同步时间
    mtimeMs?: number, // 文件夹最后同步时间
    expiredAt?: number, // 时效过期时间
}

type WebPageProps = {
    deleted: boolean,
    icon: string,
    title: string,
    version: string,
    description: string,
}

type WebPageDatas = {
    plainData: PlainData,
}

type WebPage = WebPageIds & WebPageTimes & WebPageDatas & WebPageProps;

type UpdateProps<T,Key extends keyof T> = {[key in Key]?: T[key]}
type AllowUpdateKeys = keyof  WebPageDatas | keyof  WebPageProps | 'url' | 'urls'

interface WebPageObject extends WebPage{
    isValid: Function, // 数据有效性校验，判断该数据是否有效
    setValue: Function,
}

class WebPageItem implements WebPageObject{
    createAt: number;
    deleted: boolean;
    description: string;
    expiredAt: number;
    icon: string;
    key: string;
    lastSyncTime: number;
    mtimeMs: number;
    plainData: PlainData;
    title: string;
    updateAt: number;
    url: string;
    urls: string[];
    version: string;

    readonly initHash:string;

    constructor(webPage:WebPage) {
        this.initHash = this.setValue(webPage)
    }

    setValue(webPage:UpdateProps<WebPage, AllowUpdateKeys>):string{
        for(let i in webPage){
            // @ts-ignore
            if(webPage[i]!==undefined){
                // @ts-ignore
                this[i] = webPage[i]
            }
        }
        this.updateAt = Date.now();
        return this.getDataHash();
    }

    isValid() {
        const {plainData,expiredAt} = this;
        return expiredAt > Date.now() && plainData.steps.length > 0 && plainData.snapshots.length > 0
    }

    getDataHash():string{
        const string = JSON.stringify({
            version: this.version,
            deleted: this.deleted,
            plainData: this.plainData,
            description: this.description,
            icon: this.icon,
            urls: this.urls,
        })
        return md5(string)
    }
}

export type {
    PlainData,
    WebPage,
    WebPageItem,
}
